;	===========================================
;	Text-Clock with ATmega8515
;	===========================================
;	A0: Output LED - Segment F
;	A1: Output LED - Segment G
;	A2: Output LED - Segment A1
;	A3: Output LED - Segment H
;	A4: Output LED - Segment J
;	A5: Output LED - Segment A2
;	A6: Output LED - Segment B
;	A7: Output LED - Segment K
;	-------------------------------
;	B0: Output 1 of 10 Decoder a
;	B1: Output 1 of 10 Decoder b
;	B2: Output 1 of 10 Decoder c
;	B3: Output 1 of 10 Decoder d
;	B4: Output LED - Segment DP
;	B5: Input MOSI
;	B6: Output MISO
;	B7: Input SCK
;	-------------------------------
;	C0: Output LED - Segment P
;	C1: Output LED - Segment E
;	C2: Output LED - Segment D1
;	C3: Output LED - Segment N
;	C4: Output LED - Segment M
;	C5: Output LED - Segment D2
;	C6: Output LED - Segment L
;	C7: Output LED - Segment C
;	-------------------------------
;	D0: Input DCF-Receiver
;	D1: Input Alarm
;	D2: Input free
;	D3: Input free
;	D4: Input Language DE/EN
;	D5: Input DateText dd.mm.yyyy / dd. Mon.yy
;	D6: Input Nightmode On/Off
;	D7: Input free
;	-------------------------------
;	E0: Input free
;	E1: Input free
;	E2: Input free
;
;	-------------------------------
;	Timers
;	-------------------------------
;	Timer0: Interrupt every 0,1ms (Prescaler 8, Compare 50 @ 4MHz)
;	Timer1: Interrupt every 10ms (Prescaler 8, Compare 5000 @ 4MHz)
;
;	-----------------------------------
;	Standard definitions for ATmega8515
;	-----------------------------------
.include "m8515def.inc"

;	-------------------------------
;	SRAM
;	-------------------------------
.dseg
.org	0x60

timbeg:	.byte	16		;time data
datbeg:	.byte	16		;date data
dowbeg:	.byte	16		;day of week data
alabeg:	.byte	16		;alarm data
errbeg:	.byte	16		;error data
dcfbeg:	.byte	64		;DCF data
;	-------------------------------
;	Constant definitions
;	-------------------------------
.equ	rambeg	=0x60		;RAM start
.equ	t0end	=50		;Timer0 end value 0,1ms
.equ	t1end	=5000		;Timer1 end value 10ms
.equ	t	=6		;t-flag position in SREG
.equ	dp	=4		;output decimal point in ctrlport
.equ	dispm	=8		;number of 16 segment displays
.equ	dcfsig	=0		;input DCF receiver
.equ	alasig	=1		;input alert mode
.equ	langm	=4		;input Language select
.equ	datexm	=5		;input DateTextMode select
.equ	nightm	=6		;input NightMode select
.equ	brigm	=16		;max brightness PWM
.equ	dcfimp	=59		;max DCF pulses (59)
.equ	tdcf0	=5		;*10 ms DCF logical zero time
.equ	tdcf1	=16		;*10 ms DCF logical one time
.equ	tdcfm	=160		;*10 ms DCF mark for new time frame
.equ	timeout	=250		;*10 ms DCF timeout (no signal)
.equ	waiting	=250		;*10 ms waiting time for logo
.equ	ctrlport=portb
.equ	ctrlddr	=ddrb
.equ	ctrlpin	=pinb
.equ	led1port=porta
.equ	led1ddr	=ddra
.equ	led1pin	=pina
.equ	led2port=portc
.equ	led2ddr	=ddrc
.equ	led2pin	=pinc
.equ	dcfport	=portd
.equ	dcfddr	=ddrd
.equ	dcfpin	=pind
;	-------------------------------
;	Register definitions
;	-------------------------------
.def	stor	=r0		;storage variable for LPM
.def	flags	=r2		;flag storage
.def	bright	=r3		;actual display brightness
.def	setbrg	=r4		;set display brightness
.def	dots	=r5		;decimal points
.def	sect	=r6		;seconds tens
.def	secu	=r7		;seconds units
.def	hourt	=r8		;hour tens
.def	houru	=r9		;hour units
.def	mont	=r10		;month tens
.def	monu	=r11		;month units
.def	dowu	=r12		;day of week units
.def	temp	=r16		;temporary variable
.def	temp2	=r17		;temporary variable 2
.def	temp3	=r18		;temporary variable 3
.def	temp4	=r19		;temporary variable 4
.def	tcount	=r20		;temporary counter variable
.def	titemp	=r21		;timer ISR temporary variable
.def	dcfloop	=r22		;loop for DCF-Storage
.def	disco	=r23		;display counter
.def	mode	=r24		;display mode 0=Time,1=Weekday,2=Date,3=Error
.def	tico10	=r25		;time counter increment every 10 ms

;	---------------------------------
;	Program start, Interrupt vectors
;	---------------------------------
.cseg
.org	0x0000

reset:	rjmp	initia		;program start
exint0:	reti
exint1:	reti
t1icap:	reti
t1coma:	rjmp	timer1		;time counter
t1comb:	reti
t1ovfl:	reti
t0ovfl:	reti
spiint:	reti
uartrx:	reti
uartem:	reti
uarttx:	reti
ancomp:	reti
exint2:	reti
t0comp:	rjmp	timer0		;display handling
ee_rdy:	reti
spmrdy:	reti

;Initialize RAM, I/O and interrupts
initia:	cli				;disable interrupts

	clr	temp
	clr	temp2
	out	ddre, temp		;set inputs
	ldi	temp, 0xff
	out	porte, temp		;set pull-ups

	clr	temp
	out	dcfddr, temp		;set inputs
	ldi	temp, 0xff
	out	dcfport, temp		;set pull-ups

	ldi	temp, 0xff
	out	led1ddr, temp		;set outputs
	out	led2ddr, temp
	clr	temp
	out	led1port,temp		;clear ports
	out	led2port,temp

	ldi	temp, 0x1f
	out	ctrlddr, temp		;set in/outputs
	ldi	temp, 0xe0
	out	ctrlport,temp		;set pull-ups

	ldi	temp,0xb2		;calibrate oscillator
	out	osccal,temp
	ldi	temp,low(ramend)	;set stack pointer
	out	spl,temp
	ldi	temp,high(ramend)
	out	sph,temp

	ldi	yl,low(rambeg)		;RAM start low
	ldi	yh,high(rambeg)		;RAM start high
init:	st	y+,temp2		;clear RAM
	cpi	yl,low(ramend+1)	;RAM end low?
	brne	init			;no -> loop
	cpi	yh,high(ramend+1)	;RAM end high?
	brne	init			;no -> loop

	ldi	temp,t0end		;Timer0 end value
	out	ocr0,temp
	ldi	temp,high(t1end)	;Timer1 end value a H
	out	ocr1ah,temp
	ldi	temp,low(t1end)		;Timer1 end value a L
	out	ocr1al,temp
	ldi	temp,(1<<wgm01)+(1<<cs01);Timer0 clear afer compare, prescaler = 8
	out	tccr0,temp
	out	tccr1a,temp2		;Timer1 no PWM, no Compare-Out
	ldi	temp,(1<<wgm12)+(1<<cs11);Timer1 clear afer compare, prescaler = 8
	out	tccr1b,temp
	out	gimsk,temp2		;disable external interrupts
	ldi	temp,(1<<ocie0)+(1<<ocie1a);Timer1 compare1
	out	timsk,temp
	sei				;enable interrupts

	rcall	alert			;load Alarm text
	rcall	logo			;show logo
	ldi	temp,brigm
	mov	setbrg,temp
	clr	tico10
wait:	cpi	tico10,waiting		;wait
	brlo	wait

;Main program
main:	clr	dcfloop			;jump here every minute
	clr	sect
	clr	secu
	ldi	xl,low(dcfbeg)
	ldi	xh,high(dcfbeg)

;duration low signal
start:	clr	tico10			;jump here every second
ifhigh:	cpi	tico10,timeout
	brlo	ifhi2
	rjmp	errdcf0			;short circuit
ifhi2:	brtc	ifhigh			;wait while DCF (t-flag) low
	cpi	tico10,tdcfm
	brsh	main			;minute mark, no error

;duration high signal
	clr	tico10
iflow:	cpi	tico10,timeout
	brlo	iflo2
	rjmp	errdcf1			;broken wire
iflo2:	brts	iflow			;wait while DCF (t-flag) high
	cpi	tico10,tdcf1
	brsh	dcf1			;logical one
	cpi	tico10,tdcf0
	brsh	dcf0			;logical zero
	rjmp	errsyn			;spike pulse error

dcf0:	clr	temp
	rjmp	main2
dcf1:	ldi	temp,1

main2:	st	x+,temp			;store DCF pulses
	cpi	mode,3
	breq	mame
	cpi	dcfloop,10		;5 sec. day of week
	brne	mam1
	ldi	mode,1
mam1:	cpi	dcfloop,15
	brne	mam2
	ldi	mode,0
mam2:	cpi	dcfloop,40		;5 sec. date
	brne	mam3
	ldi	mode,2
mam3:	cpi	dcfloop,45
	brne	mame
	ldi	mode,0
mame:	nop

sec1:	inc	secu			;count second ones
	ldi	temp,10
	cp	secu,temp
	brlo	sece
	clr	secu

sec2:	inc	sect			;count second tens
	ldi	temp,6
	cp	sect,temp
	brlo	sece
	clr	sect

sece:	ldi	temp,0x30		;load second tens
	add	temp,sect
	rcall	fmread
	sts	timbeg+12,temp
	sts	timbeg+13,temp2

	ldi	temp,0x30		;load second ones
	add	temp,secu
	rcall	fmread
	sts	timbeg+14,temp
	sts	timbeg+15,temp2

	inc	dcfloop			;increment dcfloop
	cpi	dcfloop,dcfimp		;for all 59 pulses
	brlo	start

main3:	clr	temp4			;check parity
	ldi	temp,21
	ldi	temp2,8
	rcall	parity
	tst	temp4
	brne	errpar
	ldi	temp,29
	ldi	temp2,7
	rcall	parity
	tst	temp4
	brne	errpar
	ldi	temp,36
	ldi	temp2,23
	rcall	parity
	tst	temp4
	breq	main4

errpar:	ser	temp			;parity error => all dots on
	mov	dots,temp
	rjmp	main

main4:	ldi	mode,0			;combine bits to bytes
	ldi	zh,high(dcfbeg)
	ldi	zl,low(dcfbeg)

time:	ldi	temp,'0'		;load ASCII
	rcall	fmread
	sts	timbeg+14,temp		;second ones
	sts	timbeg+15,temp2

	ldi	temp,'0'		;load ASCII
	rcall	fmread
	sts	timbeg+12,temp		;second tens
	sts	timbeg+13,temp2

	adiw	zh:zl,25		;minute
	clr	temp
	ld	temp2,-z
	add	temp,temp2
	rol	temp
	ld	temp2,-z
	add	temp,temp2
	rol	temp
	ld	temp2,-z
	add	temp,temp2
	rol	temp
	ld	temp2,-z
	add	temp,temp2
	ldi	temp2,0x30		;convert to ASCII
	add	temp,temp2
	rcall	fmread
	sts	timbeg+10,temp		;minute ones
	sts	timbeg+11,temp2

	adiw	zh:zl,28
	clr	temp
	ld	temp2,-z
	add	temp,temp2
	rol	temp
	ld	temp2,-z
	add	temp,temp2
	rol	temp
	ld	temp2,-z
	add	temp,temp2
	ldi	temp2,0x30		;convert to ASCII
	add	temp,temp2
	rcall	fmread
	sts	timbeg+8,temp		;minute tens
	sts	timbeg+9,temp2

	adiw	zh:zl,33		;hour
	clr	temp
	ld	temp2,-z
	add	temp,temp2
	rol	temp
	ld	temp2,-z
	add	temp,temp2
	rol	temp
	ld	temp2,-z
	add	temp,temp2
	rol	temp
	ld	temp2,-z
	add	temp,temp2
	mov	houru,temp
	ldi	temp2,0x30		;convert to ASCII
	add	temp,temp2
	rcall	fmread
	sts	timbeg+6,temp		;hour ones
	sts	timbeg+7,temp2

	adiw	zh:zl,35
	clr	temp
	ld	temp2,-z
	add	temp,temp2
	rol	temp
	ld	temp2,-z
	add	temp,temp2
	mov	hourt,temp
	ldi	temp2,0x30		;convert to ASCII
	add	temp,temp2
	rcall	fmread
	sts	timbeg+4,temp		;hour tens
	sts	timbeg+5,temp2
	clr	temp
	sts	timbeg+0,temp
	sts	timbeg+1,temp
	sts	timbeg+2,temp
	sts	timbeg+3,temp

	adiw	zh:zl,45		;day of week
	clr	temp
	ld	temp2,-z
	add	temp,temp2
	rol	temp
	ld	temp2,-z
	add	temp,temp2
	rol	temp
	ld	temp2,-z
	add	temp,temp2
	mov	dowu,temp

dow:	clr	temp
	ldi	xl,low(dowbeg)
	ldi	xh,high(dowbeg)

	sbis	dcfpin,langm		;text language
	rjmp	dowe
dowd:	ldi	zl,low(tdowde*2)
	ldi	zh,high(tdowde*2)
	rjmp	dow1
dowe:	ldi	zl,low(tdowen*2)
	ldi	zh,high(tdowen*2)

dow1:	inc	temp
	cp	temp,dowu
	brsh	dow2
	adiw	zh:zl,8
	rjmp	dow1
dow2:	rcall	writel

date:	ldi	zh,high(dcfbeg)		;date
	ldi	zl,low(dcfbeg)

	adiw	zh:zl,54		;year
	clr	temp
	ld	temp2,-z
	add	temp,temp2
	rol	temp
	ld	temp2,-z
	add	temp,temp2
	rol	temp
	ld	temp2,-z
	add	temp,temp2
	rol	temp
	ld	temp2,-z
	add	temp,temp2
	ldi	temp2,0x30		;convert to ASCII
	add	temp,temp2
	rcall	fmread
	sts	datbeg+14,temp		;year ones
	sts	datbeg+15,temp2

	adiw	zh:zl,58
	clr	temp
	ld	temp2,-z
	add	temp,temp2
	rol	temp
	ld	temp2,-z
	add	temp,temp2
	rol	temp
	ld	temp2,-z
	add	temp,temp2
	rol	temp
	ld	temp2,-z
	add	temp,temp2
	ldi	temp2,0x30		;convert to ASCII
	add	temp,temp2
	rcall	fmread
	sts	datbeg+12,temp		;year tens
	sts	datbeg+13,temp2

	sbis	dcfpin,datexm
	rjmp	mon

	ldi	temp,'0'		;load ASCII
	rcall	fmread
	sts	datbeg+10,temp		;year hundreds
	sts	datbeg+11,temp2

	ldi	temp,'2'		;load ASCII
	rcall	fmread
	sts	datbeg+8,temp		;year thousands
	sts	datbeg+9,temp2

mon:	adiw	zh:zl,49		;month
	clr	temp
	ld	temp2,-z
	add	temp,temp2
	rol	temp
	ld	temp2,-z
	add	temp,temp2
	rol	temp
	ld	temp2,-z
	add	temp,temp2
	rol	temp
	ld	temp2,-z
	add	temp,temp2
	mov	monu,temp
	ldi	temp2,0x30		;convert to ASCII
	add	temp,temp2
	rcall	fmread
	sts	datbeg+6,temp		;month ones
	sts	datbeg+7,temp2

	adiw	zh:zl,49
	ld	temp,z
	mov	mont,temp
	ldi	temp2,0x30		;convert to ASCII
	add	temp,temp2
	rcall	fmread
	sts	datbeg+4,temp		;month tens
	sts	datbeg+5,temp2

	sbic	dcfpin,datexm		;text month?
	rjmp	day

	ldi	temp,10
	mul	mont,temp
	add	monu,stor
	clr	temp
	ldi	xl,low(datbeg+4)
	ldi	xh,high(datbeg+4)

	sbis	dcfpin,langm		;text language
	rjmp	mone
mond:	ldi	zl,low(tmonde*2)
	ldi	zh,high(tmonde*2)
	rjmp	mon1
mone:	ldi	zl,low(tmonen*2)
	ldi	zh,high(tmonen*2)

mon1:	inc	temp
	cp	temp,monu
	brsh	mon2
	adiw	zh:zl,4
	rjmp	mon1
mon2:	rcall	wrmonl

day:	ldi	zh,high(dcfbeg)		;day
	ldi	zl,low(dcfbeg)
	adiw	zh:zl,40
	clr	temp
	ld	temp2,-z
	add	temp,temp2
	rol	temp
	ld	temp2,-z
	add	temp,temp2
	rol	temp
	ld	temp2,-z
	add	temp,temp2
	rol	temp
	ld	temp2,-z
	add	temp,temp2
	ldi	temp2,0x30		;convert to ASCII
	add	temp,temp2
	rcall	fmread
	sts	datbeg+2,temp		;day ones
	sts	datbeg+3,temp2

	adiw	zh:zl,42
	clr	temp
	ld	temp2,-z
	add	temp,temp2
	rol	temp
	ld	temp2,-z
	add	temp,temp2
	ldi	temp2,0x30		;convert to ASCII
	add	temp,temp2
	rcall	fmread
	sts	datbeg+0,temp		;day tens
	sts	datbeg+1,temp2

dimm:	sbis	dcfpin,nightm		;nightmode override
	rjmp	dimd
	mov	temp,hourt		;day?
	cpi	temp,1
	brsh	dimm2
	mov	temp,houru
	cpi	temp,6
	brsh	dimm2
	rjmp	dimn

dimm2:	mov	temp,hourt		;night?
	cpi	temp,2
	brlo	dimd
	mov	temp,houru
	cpi	temp,2
	brlo	dimd

dimn:	ldi	temp,1			;night brightness
	mov	setbrg,temp
	rjmp	main

dimd:	ldi	temp,brigm		;day brightness
	mov	setbrg,temp
	rjmp	main

;Timer0 ISR: use disco, titemp and pointer y only here
timer0:	in	flags,sreg		;save flags
	in	titemp,dcfpin		;store DCF status
	bst	titemp,dcfsig		;in T-Flag
	bld	flags,t

	clr	titemp			;clear ports
	out	led1port,titemp
	out	led2port,titemp
	cbi	ctrlport,dp
	sbi	ctrlport,3

	inc	disco			;increment display counter
	cpi	disco,dispm		;while disco < DispMax
	brlo	goe

	clr	disco			;clear disco

	inc	bright			;brightness counter
	ldi	titemp,brigm
	cp	bright,titemp
	brlo	go1
	clr	bright

go1:	cpi	mode,0
	brne	go2
	ldi	titemp,0b00011110	;set points
	mov	dots,titemp
	ldi	yh,high(timbeg)
	ldi	yl,low(timbeg)

go2:	cpi	mode,1
	brne	go3
	ldi	titemp,0		;clear points
	mov	dots,titemp
	ldi	yh,high(dowbeg)
	ldi	yl,low(dowbeg)

go3:	cpi	mode,2
	brne	go4
	sbic	dcfpin,datexm
	ldi	titemp,0b01010000	;set points
	sbis	dcfpin,datexm
	ldi	titemp,0b01000100	;set points
	mov	dots,titemp
	ldi	yh,high(datbeg)
	ldi	yl,low(datbeg)

go4:	cpi	mode,3
	brne	go5
	ldi	yh,high(errbeg)
	ldi	yl,low(errbeg)

go5:	sbis	dcfpin,alasig
	rjmp	go6
	ldi	titemp,0		;clear points
	mov	dots,titemp
	ldi	yh,high(alabeg)
	ldi	yl,low(alabeg)

go6:	adiw	yh:yl,16

goe:	ldi	titemp,0xe0		;set pull-ups
	or	titemp,disco		;and digit
	out	ctrlport,titemp

dpu1:	brtc	dpu2			;dot shows DCF status
	mov	titemp,dots
	sbr	titemp,128
	mov	dots,titemp
	rjmp	pwm
dpu2:	mov	titemp,dots
	cbr	titemp,128
	mov	dots,titemp

pwm:	tst	setbrg			;brightness via PWM
	breq	pwm0
	ldi	titemp,brigm
	cp	setbrg,titemp
	breq	dots
	cp	bright,setbrg
	brlo	dots
	rjmp	pwm0

dots:	mov	titemp,dots		;dot bits
dps0:	cpi	disco,0
	brne	dps1
	sbrc	titemp,0
	rjmp	dpset
	sbrs	titemp,0
	rjmp	dpres
dps1:	cpi	disco,1
	brne	dps2
	sbrc	titemp,1
	rjmp	dpset
	sbrs	titemp,1
	rjmp	dpres
dps2:	cpi	disco,2
	brne	dps3
	sbrc	titemp,2
	rjmp	dpset
	sbrs	titemp,2
	rjmp	dpres
dps3:	cpi	disco,3
	brne	dps4
	sbrc	titemp,3
	rjmp	dpset
	sbrs	titemp,3
	rjmp	dpres
dps4:	cpi	disco,4
	brne	dps5
	sbrc	titemp,4
	rjmp	dpset
	sbrs	titemp,4
	rjmp	dpres
dps5:	cpi	disco,5
	brne	dps6
	sbrc	titemp,5
	rjmp	dpset
	sbrs	titemp,5
	rjmp	dpres
dps6:	cpi	disco,6
	brne	dps7
	sbrc	titemp,6
	rjmp	dpset
	sbrs	titemp,6
	rjmp	dpres
dps7:	cpi	disco,7
	brne	dpend
	sbrc	titemp,7
	rjmp	dpset
	sbrs	titemp,7
	rjmp	dpres

dpset:	sbi	ctrlport,dp		;dot port
	rjmp	dpend
dpres:	cbi	ctrlport,dp

dpend:	ld	titemp,-y		;leds data
	out	led2port,titemp
	ld	titemp,-y
	out	led1port,titemp
	rjmp	tend

pwm0:	ld	titemp,-y		;leds off PWM time
	ld	titemp,-y
	clr	titemp
	out	led2port,titemp
	out	led1port,titemp
	cbi	ctrlport,dp

tend:	out	sreg,flags		;restore flags
	reti

;Timer1 ISR: time counting
timer1:	in	flags,sreg		;save flags
	inc	tico10			;increment time counter
	out	sreg,flags		;restore flags
	reti

;Parity check
;temp  : in DCF offset
;temp2 : in number of bits
;temp4 : out status
;Intern: tcount,temp3,z-pointer
parity:	clr	temp3
	clr	tcount
	ldi	zh,high(dcfbeg)
	ldi	zl,low(dcfbeg)
	add	zl,temp
	adc	zh,temp3
pari2:	ld	temp,z+			;add bits
	add	temp3,temp
	inc	tcount
	cp	tcount,temp2
	brlo	pari2
	sbrc	temp3,0			;even?
	inc	temp4
	ret

;FLASH Memory read
;temp  : in ASCII character
;temp  : out 16 segment data
;temp2 : out 16 segment data
;Intern: z-pointer
fmread:	ldi	zh,high(segmtb*2)
	ldi	zl,low(segmtb*2)
	subi	temp,0x20	;ASCII to value
	clr	temp2
	rol	temp		;*2
	add	zl,temp
	adc	zh,temp2
	lpm	temp,z+
	lpm	temp2,z

	ldi	zh,high(dcfbeg)
	ldi	zl,low(dcfbeg)
	ret

;Write a line on display
;x-pointer : write RAM (timbeg or alabeg)
;z-pointer : read FLASH
;Intern: temp,tcount,temp3,temp4
writel:	clr	tcount

write2:	lpm	temp,z+
	mov	temp3,zl
	mov	temp4,zh
	rcall	fmread
	mov	zl,temp3
	mov	zh,temp4
	st	x+,temp
	st	x+,temp2
	inc	tcount
	cpi	tcount,dispm
	brlo	write2
	ret

;Write month on display
;x-pointer : write RAM (4 digits)
;z-pointer : read FLASH
;Intern: temp,tcount,temp3,temp4
wrmonl:	clr	tcount

wrmon2:	lpm	temp,z+
	mov	temp3,zl
	mov	temp4,zh
	rcall	fmread
	mov	zl,temp3
	mov	zh,temp4
	st	x+,temp
	st	x+,temp2
	inc	tcount
	cpi	tcount,4
	brlo	wrmon2
	ret

; ----- Messages -----

errdcf0:ldi	xl,low(errbeg)		;DCF low (short circuit)
	ldi	xh,high(errbeg)
	ldi	zl,low(txdcf0*2)
	ldi	zh,high(txdcf0*2)
	rcall	writel
	clr	dots			;clear points
	ldi	mode,3
	rjmp	main

errdcf1:ldi	xl,low(errbeg)		;DCF high (broken wire)
	ldi	xh,high(errbeg)
	ldi	zl,low(txdcf1*2)
	ldi	zh,high(txdcf1*2)
	rcall	writel
	clr	dots			;clear points
	ldi	mode,3
	rjmp	main

errsyn:	ldi	xl,low(errbeg)		;No sync
	ldi	xh,high(errbeg)
	ldi	zl,low(txsync*2)
	ldi	zh,high(txsync*2)
	rcall	writel
	clr	dots			;clear points
	ldi	mode,3
	rjmp	main

logo:	ldi	xl,low(errbeg)		;Logo
	ldi	xh,high(errbeg)
	ldi	zl,low(txlogo*2)
	ldi	zh,high(txlogo*2)
	rcall	writel
	ldi	temp,0b01100111		;set points
	mov	dots,temp
	ldi	mode,3
	ret

alert:	ldi	xl,low(alabeg)		;Alarm
	ldi	xh,high(alabeg)
	sbis	dcfpin,langm		;text language
	rjmp	alere
	ldi	zl,low(txald*2)
	ldi	zh,high(txald*2)
	rjmp	alend
alere:	ldi	zl,low(txale*2)
	ldi	zh,high(txale*2)
alend:	rcall	writel
	ret

txdcf0:	.db	"DCF_ER_0"
txdcf1:	.db	"DCF_ER_1"
txsync:	.db	"NO_SYNC!"
txlogo:	.db	"**OLAF**"
txald:	.db	"*ALARM!*"
txale:	.db	"*ALERT!*"

tdowde:	.db	" MONTAG "
	.db	"DIENSTAG"
	.db	"MITTWOCH"
	.db	"DONNERST"
	.db	"FREITAG "
	.db	"SAMSTAG "
	.db	"SONNTAG "

tdowen:	.db	" MONDAY "
	.db	"TUESDAY "
	.db	"WEDNESDA"
	.db	"THURSDAY"
	.db	" FRIDAY "
	.db	"SATURDAY"
	.db	" SUNDAY "

tmonde:	.db	" JAN"
	.db	" FEB"
	.db	" MRZ"
	.db	" APR"
	.db	" MAI"
	.db	" JUN"
	.db	" JUL"
	.db	" AUG"
	.db	" SEP"
	.db	" OKT"
	.db	" NOV"
	.db	" DEZ"

tmonen:	.db	" JAN"
	.db	" FEB"
	.db	" MAR"
	.db	" APR"
	.db	" MAY"
	.db	" JUN"
	.db	" JUL"
	.db	" AUG"
	.db	" SEP"
	.db	" OCT"
	.db	" NOV"
	.db	" DEC"

segmtb:	;	Port A top Port C bot
	;	  kbAjhagf   clDmndep
	.db	0b00000000,0b00000000	;Code 20: Space
	.db	0b00111000,0b00100000	;Code 21: !
	.db	0b01001000,0b00000000	;Code 22: "
	.db	0b11001000,0b10110101	;Code 23: #
	.db	0b10101101,0b10110101	;Code 24: $
	.db	0b10011101,0b10111001	;Code 25: %
	.db	0b00001110,0b01100111	;Code 26: &
	.db	0b00001000,0b00000000	;Code 27: '
	.db	0b00010000,0b01000000	;Code 28: (
	.db	0b00000010,0b00001000	;Code 29: )
	.db	0b10011010,0b01011001	;Code 2A: *
	.db	0b10001000,0b00010001	;Code 2B: +
	.db	0b00000000,0b00001000	;Code 2C: ,
	.db	0b10000000,0b00000001	;Code 2D: -
	.db	0b00000000,0b00000100	;Code 2E: .
	.db	0b00010000,0b00001000	;Code 2F: /

	;	Port A top Port C bot
	;	  kbAjhagf   clDmndep
	.db	0b01110101,0b10101110	;Code 30: 0
	.db	0b01010000,0b10000000	;Code 31: 1
	.db	0b11100100,0b00101100	;Code 32: 2
	.db	0b11100100,0b10100101	;Code 33: 3
	.db	0b11000001,0b10000001	;Code 34: 4
	.db	0b00100101,0b01100101	;Code 35: 5
	.db	0b10100101,0b10100111	;Code 36: 6
	.db	0b01100100,0b10000000	;Code 37: 7
	.db	0b11100101,0b10100111	;Code 38: 8
	.db	0b11100101,0b10100101	;Code 39: 9
	.db	0b00000100,0b00000100	;Code 3A: :
	.db	0b00000100,0b00001000	;Code 3B: ;
	.db	0b00010000,0b01000001	;Code 3C: <
	.db	0b10000000,0b00100101	;Code 3D: =
	.db	0b10000010,0b00001000	;Code 3E: >
	.db	0b11100100,0b00010000	;Code 3F: ?

	;	Port A top Port C bot
	;	  kbAjhagf   clDmndep
	.db	0b11101101,0b00100110	;Code 40: @
	.db	0b11100101,0b10000011	;Code 41: A
	.db	0b11101100,0b10110100	;Code 42: B
	.db	0b00100101,0b00100110	;Code 43: C
	.db	0b01101100,0b10110100	;Code 44: D
	.db	0b00100101,0b00100111	;Code 45: E
	.db	0b00100101,0b00000011	;Code 46: F
	.db	0b10100101,0b10100110	;Code 47: G
	.db	0b11000001,0b10000011	;Code 48: H
	.db	0b00101100,0b00110100	;Code 49: I
	.db	0b01000000,0b10100110	;Code 4A: J
	.db	0b00010001,0b01000011	;Code 4B: K
	.db	0b00000001,0b00100110	;Code 4C: L
	.db	0b01010011,0b10000010	;Code 4D: M
	.db	0b01000011,0b11000010	;Code 4E: N
	.db	0b01100101,0b10100110	;Code 4F: O

	;	Port A top Port C bot
	;	  kbAjhagf   clDmndep
	.db	0b11100101,0b00000011	;Code 50: P
	.db	0b01100101,0b11100110	;Code 51: Q
	.db	0b11100101,0b01000011	;Code 52: R
	.db	0b10100101,0b10100101	;Code 53: S
	.db	0b00101100,0b00010000	;Code 54: T
	.db	0b01000001,0b10100110	;Code 55: U
	.db	0b00010001,0b00001010	;Code 56: V
	.db	0b01000001,0b11001010	;Code 57: W
	.db	0b00010010,0b01001000	;Code 58: X
	.db	0b00010010,0b00010000	;Code 59: Y
	.db	0b00110100,0b00101100	;Code 5A: Z
	.db	0b00101000,0b00110000	;Code 5B: [
	.db	0b00000010,0b01000000	;Code 5C: \;
	.db	0b00001100,0b00010100	;Code 5D: ]
	.db	0b00000000,0b01001000	;Code 5E: ^
	.db	0b00000000,0b00100100	;Code 5F: _
